/**
 *
 * Copyright (C) 2009-2014 INRA.
 * Copyright (C) 2009-2014 CETIOM.
 *
 *
 * TODO licence
 */

// @@tagdynamic@@
// @@tagdepends: vle.discrete-time @@endtagdepends


#include <math.h>
#include <vle/DiscreteTime.hpp>
#include <sunflo_utils.hpp>
#include <Phenologie.hpp>
#include <ParametresPlante.hpp>
#include <ParametresVariete.hpp>
#include <ParametresRendement.hpp>

namespace sunflo {

using namespace vle::discrete_time;

class Diagnostic : public DiscreteTimeDyn
{
public :
    ParametresVariete pv;
    ParametresPlante pp;
    ParametresRendement pr;

    /*Sync*/ Var TT_A2;
    /*Sync*/ Var ETRETM;

////////////////////////////////////////////////////////////////////////
/////////////////////////////REMI ADDS//////////////////////////////////
////////////////////////////////////////////////////////////////////////

    /*Sync*/ Var PhasePhenoPlante;
    /*Sync*/ Var Tmoy;
    /*Sync*/ Var Pluie;
    /*Sync*/ Var Ei;
    /*Sync*/ Var FHTR;
    /*Sync*/ Var CT_RUE;



// Covariables to compute OC

    /// # Jours entre maturation et récolte.
    Var D_MH;
   /// # Jours entre semis et germination.
    Var D_SE;
   /// # Jours de phase juvénile et croissance active.
    Var D_EF;

    /// # Somme ETPET entre maturation et récolte.
    Var SET_MH;

    /// # Moyenne ETPET entre maturation et récolte.
    Var MET_MH;

   /// # Somme ETPET durant les jours de phase juvénile et croissance active.
    Var SET_EF;

    /// # Moyenne ETPET durant les jours de phase juvénile et croissance active.
    Var MET_EF;

    // Thermal stress entre germinatioan et initiation florale.
    Var NHT28_EF;
 
    // Thermal stress during whole season
    Var NHT28;

    // Radiations
    Var RG;
    // Sum of radiations during flowering
     Var SGR_FM;


// Covariables to compute GY

// Somme précipitations durant la phase végétative (covariables statistiques)
    Var SRR_EF;
// Somme précipitations durant la floraison  (covariables statistiques)
    Var SRR_FM;
// Somme radiations interceptées durant la phase végétative (covariables statistiques)
    Var SIR_EF;
// Somme  radiations interceptées durant la floraison  (covariables statistiques)
    Var SIR_FM;
// Effet contrainte hydrique sur la photosynthèse durant la phase végétative (covariables statistiques)
    Var SFHTR_EF;
//  Effet contrainte hydrique sur la photosynthèse durant la floraison  (covariables statistiques)
    Var SFHTR_FM;

// Facteur de réponse de la photosynthèse à la contrainte azote
    Var FNIRUE;
//  Effet contrainte azote  sur la photosynthèse après la maturation  (covariables statistiques)
    Var SFNRUE_MH;

// Somme hautes températures durant la floraison
    Var SHT_FM; 

    Diagnostic(const vle::devs::DynamicsInit& model,
            const vle::devs::InitEventList& events) :
                DiscreteTimeDyn(model, events)
    {
        pv.initialiser(events);
        pr.initialiser(events);
        pp.initialiser(events);

        TT_A2.init(this, "TT_A2", events);
        ETRETM.init(this, "ETRETM", events);

	
	D_MH.init(this, "D_MH" , events);
	D_SE.init(this, "D_SE", events);
	D_EF.init(this, "D_EF", events);

        SET_MH.init(this, "SET_MH" , events);
        MET_MH.init(this, "MET_MH" , events);

        SET_EF.init(this, "SET_EF" , events);
        MET_EF.init(this, "MET_EF" , events);

	NHT28_EF.init(this, "NHT28_EF", events);
	NHT28.init(this, "NHT28", events);
 
	SGR_FM.init(this, "SGR_FM", events);
	RG.init(this, "RG", events);

        PhasePhenoPlante.init(this, "PhasePhenoPlante", events);
	Tmoy.init(this, "Tmoy", events);
	Pluie.init(this, "Pluie", events);
	Ei.init(this,  "Ei", events);
	FHTR.init(this,  "FHTR", events);
	CT_RUE.init(this,  "CT_RUE", events);


////// Covariables GY
	SRR_EF.init(this, "SRR_EF", events);
	SRR_FM.init(this, "SRR_FM", events);

	SIR_EF.init(this, "SIR_EF", events);
	SIR_FM.init(this, "SIR_FM", events);

	SFHTR_EF.init(this, "SFHTR_EF", events);
	SFHTR_FM.init(this, "SFHTR_FM", events);
	
	FNIRUE.init(this, "FNIRUE", events);
	SFNRUE_MH.init(this, "SFNRUE_MH", events);
	SHT_FM.init(this, "SHT_FM", events);
    }

    virtual ~Diagnostic() { }

    virtual void compute(const vle::devs::Time& /*time*/ )
    {


		
            {
                if ( ( PhasePhenoPlante() != PHASEPHENOPLANTE_MATURATION )
                        // traduit condition TT_A2 < date_TT_M0
                       // || ( PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE )
                       // || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE )
		   ){
                    D_MH = D_MH(-1);
                } else {
                    D_MH = D_MH(-1) + 1;
                }
            }

            {
                if ( PhasePhenoPlante() == PHASEPHENOPLANTE_GERMINATION ){
                    // traduit condition TT_A2 = 0
                    D_SE = D_SE(-1) + 1;
                } else {
                    D_SE = D_SE(-1);
                }
            }

            {
                if ( (PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE) 
			|| (PhasePhenoPlante() > PHASEPHENOPLANTE_CROISSANCEACTIVE) ){
                    D_EF = D_EF(-1);
                } else {
                    D_EF = D_EF(-1) + 1;
                }
            }

            {
                double ddt = 0.0;
                if ( PhasePhenoPlante() != PHASEPHENOPLANTE_GERMINATION ){
                    // traduit condition TT_A2 = 0
                    ddt = 0.0;
                } else {
                    ddt = 1;
                }
                D_SE = D_SE(-1) + ddt;
            }

            {
                if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_MATURATION )
                        || ( PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE )
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE ) ){
                    // traduit condition TT_A2 = 0
                    SET_MH = SET_MH(-1);
                } else {
                    SET_MH = SET_MH(-1) + ETRETM();
                }
            }

            // calcul de MET_MH
            MET_MH = SET_MH() / D_MH();
		//MET_MH = 1.0;
            {
                if ( (PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE) 
			|| (PhasePhenoPlante() > PHASEPHENOPLANTE_CROISSANCEACTIVE) ){
                    SET_EF = SET_EF(-1);
                } else {
                    SET_EF = SET_EF(-1) + ETRETM();
                }
            }

            // calcul de MET_EF
            MET_EF = SET_EF() / D_EF();


            // calcul de NHT28_EF
            {	
                if ( Tmoy() <= 28.0 ){
                    NHT28_EF = NHT28_EF(-1);
                } else if ( (PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE) 
			|| (PhasePhenoPlante() > PHASEPHENOPLANTE_CROISSANCEACTIVE) ){
                    NHT28_EF = NHT28_EF(-1);
                } else {
                    NHT28_EF = NHT28_EF(-1) + 1;
                }

            }

            // calcul de NHT28
            {
                if ( Tmoy() <= 28.0 ){
                    NHT28 = NHT28(-1);
                } else if ((PhasePhenoPlante() >= PHASEPHENOPLANTE_DESSICATION) || (PhasePhenoPlante() <= PHASEPHENOPLANTE_GERMINATION)) {
		    NHT28 = NHT28(-1);
		}
		 else {
                    NHT28 = NHT28(-1) + 1;
		    //std::cout << "temp haute NHT28 \n";
                }

            }

            // calcul de SGR_FM
            {
                if ( PhasePhenoPlante() != PHASEPHENOPLANTE_FLORAISON){
                    SGR_FM = SGR_FM(-1);
                } else {
                    SGR_FM = SGR_FM(-1) + RG();
                }

            }



	    {
                if ( (PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE) 
			|| (PhasePhenoPlante() > PHASEPHENOPLANTE_CROISSANCEACTIVE) ){
                    // traduit condition TT_A2 > date_TT_F1
                    SRR_EF = SRR_EF(-1);
                } else {
                    SRR_EF = SRR_EF(-1) + Pluie();
                }
            }

	    {
                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_MATURATION)
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 > date_TT_F1
                    SRR_FM = SRR_FM(-1);
                } else if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_FLORAISON)
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 < date_TT_E1
                    SRR_FM = SRR_FM(-1);
                } else {
                    SRR_FM = SRR_FM(-1) + Pluie();
                }
            }
	    {
                if ( (PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE) 
			|| (PhasePhenoPlante() > PHASEPHENOPLANTE_CROISSANCEACTIVE) ){
                    // traduit condition TT_A2 < date_TT_E1
                    SIR_EF = SIR_EF(-1);
                } else {
                    SIR_EF = SIR_EF(-1) + 0.48*RG()*Ei();
                }
            }
	    {
                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_MATURATION)
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 > date_TT_F1
                    SIR_FM = SIR_FM(-1);
                } else if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_FLORAISON)
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 < date_TT_E1
                    SIR_FM = SIR_FM(-1);
                } else {
                    SIR_FM = SIR_FM(-1) + 0.48*RG()*Ei();
                }
            }
	    {
                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_MATURATION)
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 > date_TT_F1
                    SFHTR_FM = SFHTR_FM(-1);
                } else if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_FLORAISON)
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 < date_TT_E1
                    SFHTR_FM = SFHTR_FM(-1);
                } else {
                    SFHTR_FM = SFHTR_FM(-1) + (1 - FHTR());
                }
            }
	    {
                if ( (PhasePhenoPlante() < PHASEPHENOPLANTE_JUVENILE) 
			|| (PhasePhenoPlante() > PHASEPHENOPLANTE_CROISSANCEACTIVE) ){
                    // traduit condition TT_A2 < date_TT_E1
                    SFHTR_EF = SFHTR_EF(-1);
                } else {
                    SFHTR_EF = SFHTR_EF(-1) + (1 - FHTR());
                }
            }



	    {
                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_MATURATION)
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 > date_TT_F1
                    SHT_FM = SHT_FM(-1);
                } else if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_FLORAISON)
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 < date_TT_E1
                    SHT_FM = SHT_FM(-1);
                } else {
                    SHT_FM = SHT_FM(-1) + (1 - CT_RUE());
                }
            }
            {
                if ( ( PhasePhenoPlante() >= PHASEPHENOPLANTE_DESSICATION)
                        && ( PhasePhenoPlante() < PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 > date_TT_M3
                    SFNRUE_MH = SFNRUE_MH(-1);
                } else if ( ( PhasePhenoPlante() < PHASEPHENOPLANTE_MATURATION)
                        || ( PhasePhenoPlante() >= PHASEPHENOPLANTE_RECOLTEE) ){
                    // traduit condition TT_A2 < date_TT_M0
                    SFNRUE_MH = SFNRUE_MH(-1);
                } else {
                    SFNRUE_MH = SFNRUE_MH(-1) + (1 - FNIRUE());
                }
            }
    }
};

} // namespace sunflo

DECLARE_DYNAMICS(sunflo::Diagnostic);

